@@ -2,7 +2,6 @@
 
 import argparse
 import base64
-import json
 import os
 import shutil
 import tempfile
@@ -23,14 +22,16 @@ from flask import Response, jsonify, request, send_file
 from sqlitedict import SqliteDict
 from werkzeug.utils import secure_filename
 
+from arcor2 import json
 from arcor2.data import events
 from arcor2.data import rpc as arcor2_rpc
 from arcor2.data.events import PackageInfo, PackageState, ProjectException
+from arcor2.data.rpc import get_id
 from arcor2.flask import RespT, create_app, run_app
-from arcor2.package import PROJECT_PATH
 from arcor2_execution_data import EVENTS, EXPOSED_RPCS
 from arcor2_execution_data import URL as EXE_URL
 from arcor2_execution_data import rpc
+from arcor2_execution_data.package import PROJECT_PATH
 
 PORT = int(os.getenv("ARCOR2_EXECUTION_PROXY_PORT", 5009))
 SERVICE_NAME = "ARCOR2 Execution Service Proxy"
@@ -50,16 +51,24 @@ class ExecutionState(Enum):
     Pending: str = "Pending"
 
 
+@dataclass
+class SummaryProject(JsonSchemaMixin):
+    """Describes a project."""
+
+    id: str
+    name: str
+    description: str
+
+
 @dataclass
 class SummaryPackage(JsonSchemaMixin):
     """Describes execution package."""
 
     id: str
     name: Optional[str] = None
-    projectId: Optional[str] = None
-    projectName: Optional[str] = None
     created: Optional[datetime] = None
     executed: Optional[datetime] = None
+    project: Optional[SummaryProject] = None
 
 
 @dataclass
@@ -97,7 +106,7 @@ rpc_responses: Dict[int, RespQueue] = {}
 
 package_state: Optional[PackageState.Data] = None
 package_info: Optional[PackageInfo.Data] = None
-exception_message: Optional[str] = None
+exception_messages: List[str] = []
 
 
 @contextmanager
@@ -113,7 +122,6 @@ def ws_thread() -> None:  # TODO use (refactored) arserver client
 
     global package_info
     global package_state
-    global exception_message
     assert ws
 
     event_mapping: Dict[str, Type[events.Event]] = {evt.__name__: evt for evt in EVENTS}
@@ -123,6 +131,9 @@ def ws_thread() -> None:  # TODO use (refactored) arserver client
 
         data = json.loads(ws.recv())  # TODO handle WebSocketConnectionClosedException
 
+        if not isinstance(data, dict):
+            continue
+
         if "event" in data:
 
             evt = event_mapping[data["event"]].from_dict(data)
@@ -133,10 +144,10 @@ def ws_thread() -> None:  # TODO use (refactored) arserver client
                 package_state = evt.data
 
                 if package_state.state == PackageState.Data.StateEnum.RUNNING:
-                    exception_message = None
+                    exception_messages.clear()
 
             elif isinstance(evt, ProjectException):
-                exception_message = evt.data.message
+                exception_messages.append(evt.data.message)
 
         elif "response" in data:
             resp = rpc_mapping[data["response"]].Response.from_dict(data)
@@ -157,10 +168,6 @@ def call_rpc(req: arcor2_rpc.common.RPC.Request) -> arcor2_rpc.common.RPC.Respon
     return resp
 
 
-def get_id() -> int:
-    return uuid.uuid4().int
-
-
 def allowed_file(filename):
     return "." in filename and filename.rsplit(".", 1)[1].lower() == "zip"
 
@@ -446,7 +453,7 @@ def get_package(packageId: str) -> RespT:  # noqa
         archive_path = os.path.join(tmpdirname, packageId)
         shutil.make_archive(archive_path, "zip", os.path.join(PROJECT_PATH, packageId))
 
-        return send_file(archive_path + ".zip", as_attachment=True, cache_timeout=0)
+        return send_file(archive_path + ".zip", as_attachment=True, max_age=0)
 
 
 @app.route("/packages", methods=["GET"])
@@ -480,8 +487,7 @@ def get_packages() -> RespT:
         sp.created = pck.package_meta.built
         sp.executed = pck.package_meta.executed
         if pck.project_meta:
-            sp.projectId = pck.project_meta.id
-            sp.projectName = pck.project_meta.name
+            sp.project = SummaryProject(pck.project_meta.id, pck.project_meta.name, pck.project_meta.description)
         ret.append(sp.to_dict())
 
     return jsonify(ret), 200
@@ -704,8 +710,8 @@ def packages_executioninfo() -> RespT:
         ret = ExecutionInfo(ExecutionState.Pending, package_state.package_id)
     elif package_state.state == PackageState.Data.StateEnum.STOPPED:
 
-        if exception_message:
-            ret = ExecutionInfo(ExecutionState.Faulted, package_state.package_id, exception_message)
+        if exception_messages:
+            ret = ExecutionInfo(ExecutionState.Faulted, package_state.package_id, " ".join(exception_messages))
         else:
             ret = ExecutionInfo(ExecutionState.Completed, package_state.package_id)
     else:
